@@ -8,6 +8,9 @@
 ##' reduce_agegroups(seq_len(20), c(0, 5, 10))
 ##' @export
 reduce_agegroups <- function(x, limits) {
-    return(limits[findInterval(x, limits)])
+    ret <- x[NA]
+    int <- findInterval(x, limits)
+    ret[int > 0] <- limits[int[int > 0]]
+    return(ret)
 }
 
